@@ -5,7 +5,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION);
-$VERSION = '0.24'; 
+$VERSION = '0.26'; 
 
 use Carp qw(carp croak);
 use Errno qw(ETIMEDOUT ECONNRESET);
@@ -18,7 +18,6 @@ use POE::Wheel::SocketFactory;
 use POE::Filter::Stackable;
 use POE::Filter::SSL;
 use POE::Filter::Stream;
-use POE::Filter::HTTP::Parser;
 
 # Other various stuff
 use URI::Split qw(uri_split);
@@ -28,7 +27,9 @@ use Data::Dumper;
 # Crazy frame stuff
 use Protocol::WebSocket::Frame;
 
+# HTTP parsing modules
 require HTTP::Request;
+require HTTP::Response;
 
 # Global stuff for checks etc.
 my $validOpts = {
@@ -441,36 +442,37 @@ sub _socket_birth {
         $heap->{filters}->{output}->push(POE::Filter::Stream->new());
         $heap->{filters}->{input}->push(POE::Filter::Stream->new());
 
-        $heap->{filters}->{output}->push( POE::Filter::HTTP::Parser->new( type => 'client' ) );                
-        $heap->{filters}->{input}->push( POE::Filter::HTTP::Parser->new( type => 'client' ) );
-
-
         $heap->{wheel} = POE::Wheel::ReadWrite->new(
-                Handle          => $socket,
-                Driver          => POE::Driver::SysRW->new(),
-                OutputFilter    => $heap->{filters}->{output},
-                InputFilter     => $heap->{filters}->{input},
-                InputEvent      => 'socket_input',
-                ErrorEvent      => 'socket_death',
+			Handle          => $socket,
+			Driver          => POE::Driver::SysRW->new(),
+			OutputFilter    => $heap->{filters}->{output},
+			InputFilter     => $heap->{filters}->{input},
+			InputEvent      => 'socket_input',
+			ErrorEvent      => 'socket_death',
         );
 
         my $request = HTTP::Request->new(GET => '/');
         $request->protocol('HTTP/1.1');
         $request->header(
-                Upgrade                         => 'WebSocket',
-                Connection                      => 'Upgrade',
-                Host                            => $heap->{uri}->{host},
-                Origin                          => $heap->{req}->{origin},
-                'Sec-WebSocket-Key'             => $heap->{req}->{'sec-websocket-key'},
-                'Sec-WebSocket-Protocol'        => 'chat',
-                'Sec-WebSocket-Version'         => 13,
+			Upgrade                         => 'WebSocket',
+			Connection                      => 'Upgrade',
+			Host                            => $heap->{uri}->{host},
+			Origin                          => $heap->{req}->{origin},
+			'Sec-WebSocket-Key'             => $heap->{req}->{'sec-websocket-key'},
+			'Sec-WebSocket-Protocol'        => 'chat',
+			'Sec-WebSocket-Version'         => 13,
         );
+		
+		# Add a lock for when to stop reading the stream
+		$heap->{httpresp} = 1;
 
-        $heap->{wheel}->put($request);
+		# Send the request to the server
+        $heap->{wheel}->put($request->as_string());
 
         # Incase we want to investigate what we sent later.
         $heap->{_state}->{req} = $request;
 
+		# Post back a copy of the request
         $kernel->post( $heap->{parent}, $heap->{handlers}->{'connected'}, $request );
 }
 
@@ -491,24 +493,34 @@ Read data from the socket
 sub _socket_input { 
         my ($kernel,$heap,$buf) = @_[KERNEL,HEAP,ARG0];
 
-        if (ref($buf) eq 'HTTP::Response') {
+		if ( $heap->{httpresp} ) {
+			$heap->{httpbuf} .= $buf;
+			
+			if ( $heap->{httpbuf} =~ m#\r\n$#m ) {
+				# Remove the lock
+				delete $heap->{httpresp};
+			
                 # Keep a copy of the response we got incase we want to have a look at it later
                 $heap->{_state}->{res} = $buf;
-
-                # Send a copy of the handshake back to the users space
-                $kernel->post( $heap->{parent}, $heap->{handlers}->{'handshake'}, $buf );
+				
+				# Create an investigatable response object
+				my $resp = HTTP::Response->parse($heap->{httpbuf});
 
                 # Lets see if we can proceed
-                if ($buf->code() == 101) {
-                        # Lets adjust our filters
-                        $heap->{filters}->{output}->pop();
-                        $heap->{filters}->{input}->pop();
-                }
-        } else {
-                $heap->{_state}->{frame}->append($buf);
+                if ($resp->code() == 101) {
+                    # Ok connected
 
-                while (my $frame = $heap->{_state}->{frame}->next) { $kernel->post( $heap->{parent}, $heap->{handlers}->{read}, $frame ) }
-        }
+					# Send a copy of the handshake back to the users space
+					$kernel->post( $heap->{parent}, $heap->{handlers}->{'handshake'}, $resp );
+                }
+			}
+
+			return;
+		}
+
+		$heap->{_state}->{frame}->append($buf);
+
+		while (my $frame = $heap->{_state}->{frame}->next) { $kernel->post( $heap->{parent}, $heap->{handlers}->{read}, $frame ) }
 }
  
 =head1 AUTHOR
